@@ -175,7 +175,7 @@ void generic_mtrie_t<T>::rm (value_t *pipe_,
     std::list<struct iter> stack;
     unsigned char *buff = NULL;
     size_t maxbuffsize = 0;
-    struct iter it = {this, NULL, NULL, 0, 0, 0, false};
+    struct iter it = {this, NULL, NULL, 0, 0, 0, 0, false};
     stack.push_back (it);
 
     while (!stack.empty ()) {
@@ -215,8 +215,14 @@ void generic_mtrie_t<T>::rm (value_t *pipe_,
                     //  visit after the operation on the child can do the removals.
                     it.processed_for_removal = true;
                     stack.push_back (it);
-                    struct iter next = {
-                      it.node->_next.node, NULL, NULL, ++it.size, 0, 0, false};
+                    struct iter next = {it.node->_next.node,
+                                        NULL,
+                                        NULL,
+                                        ++it.size,
+                                        0,
+                                        0,
+                                        0,
+                                        false};
                     stack.push_back (next);
                     break;
                 }
@@ -245,6 +251,7 @@ void generic_mtrie_t<T>::rm (value_t *pipe_,
                           it.size + 1,
                           0,
                           0,
+                          0,
                           false};
                         stack.push_back (next);
                     }
